from PIL import Image, ImageDraw, ImageFont

def draw_text(cr, text, width=160, rgb=(0, 0, 0), mode=None):
    def get_result(lines):
        text = "\n".join(lines)
        w, h = get_textsize(text)
        min_w = min([get_textsize(line)[0] for line in lines])
        return w, -min_w, h, text
    def get_textsize(text):
        l, t, r, b = draw.textbbox((0, 0), text, font=font_12 if mode else font_24)
        return r - l, b - t
    text = text.upper()
    c, r = cr
    w, h = get_textsize(text)
    text_ = text
    indices = [i for i, c in enumerate(text) if c == " "]
    if w > width and indices:
        results = []
        for i in indices:
            lines = [text[:i], text[i + 1:]]
            results.append(get_result(lines))
        w, _, h, text_ = sorted(results)[0]
    if w > width and len(indices) >= 2:
        results = []
        for i in indices:
            for j in indices:
                if i >= j:
                    continue
                lines = [text[:i], text[i + 1:j], text[j + 1:]]
                results.append(get_result(lines))
        w, _, h, text_ = sorted(results)[0]
    if w > width and len(indices) >= 3:
        results = []
        for i in indices:
            for j in indices:
                for k in indices:
                    if i >= j or i >= k or j >= k:
                        continue
                    lines = [text[:i], text[i + 1:j], text[j + 1:k], text[k + 1:]]
                    results.append(get_result(lines))
        w, _, h, text_ = sorted(results)[0]
    if w > width and len(indices) >= 4:
        results = []
        for i in indices:
            for j in indices:
                for k in indices:
                    for l in indices:
                        if i >= j or i >= k or i >= l or j >= k or j >= l or k >= l:
                            continue
                        lines = [text[:i], text[i + 1:j], text[j + 1:k], text[k + 1:l], text[l + 1:]]
                        results.append(get_result(lines))
        w, _, h, text_ = sorted(results)[0]
    if mode == "location":
        draw.multiline_text((int((c + 1.5) * 256 - w / 2), int((r + 1.5) * 256 - 84)), text_, font=font_12, fill=rgb, align="center")
    elif mode == "person":
        draw.multiline_text((int((c + 1.5) * 256 - w / 2), int((r + 1.5) * 256 - h + 76)), text_, font=font_12, fill=rgb, align="center")
    elif mode == "req":
        draw.multiline_text((int((c + 1.5) * 256 - w / 2), int((r + 1.5) * 256 - h - 108)), text_, font=font_12, fill=rgb, align="center")
    elif mode == "note":
        draw.multiline_text((int((c + 1.5) * 256 - w / 2), int((r + 1.5) * 256 + 100)), text_, font=font_12, fill=rgb, align="center")
    else:
        draw.multiline_text((int((c + 1.5) * 256 - w / 2), int((r + 1.5) * 256 - h / 2 - 6)), text_, font=font_24, fill=rgb, align="center")

parts = open("cyberpunk2077flowchart2.txt").read().split("\n\n-----\n\n")
lines = parts[0].split("\n")
lines = [line for i, line in enumerate(lines) if i % 2 == 0]
rows = []
for i, line in enumerate(lines):
    rows.append([])
    x = 2
    while x < len(line):
        rows[i].append(line[x:x+5])
        x += 7
names = {}
persons = {}
locations = {}
notes = {}
for line in parts[1].split("\n"):
    names[line[:5]] = line[6:].split(" [")[0].split(" | ")[0]
    if "[" in line:
        persons[line[:5]] = line.split(" [")[1].split("]")[0].split(", ")[0]
        if persons[line[:5]] == "-":
            persons[line[:5]] = ""
        if ", " in line.split(" [")[1].split("]")[0]:
            locations[line[:5]] = line.split(" [")[1].split("]")[0].split(", ")[1].replace(" / ", ", ")
            if locations[line[:5]] == "-":
                locations[line[:5]] = ""
    if "|" in line:
        notes[line[:5]] = line.split("]")[-1].split(" | ")[1].split(", ")

n_rows = len(rows)
n_cols = max([len(cols) for cols in rows])

image = Image.new("RGB", ((1 + n_cols) * 256, (2 + n_rows) * 256), (255, 255, 255))
draw = ImageDraw.Draw(image)
font_36 = ImageFont.truetype("Rajdhani-Bold.ttf", 52)
font_24 = ImageFont.truetype("Rajdhani-SemiBold.ttf", 22)
font_12 = ImageFont.truetype("Rajdhani-Medium.ttf", 12)
draw.rectangle((34 * 256 + 32, 1 * 256 + 32, image.size[0] + 4, 1 * 256 + 224), fill=(255, 255, 0), outline=(0, 0, 0), width=4)
draw.rectangle((34 * 256 + 32, 10 * 256 + 32, image.size[0] + 4, 10 * 256 + 224), fill=(255, 255, 0), outline=(0, 0, 0), width=4)

for r, row in enumerate(rows):
    print(r + 1, "/", n_rows)
    for c, col in enumerate(row):
        if col == "     ":
            continue
        rgb = {
            "M": (255, 255, 0),
            "S": (0, 192, 255),
            "s": (0, 192, 255),
            "G": (224, 224, 224), # (0, 192, 0),
            "C": (224, 224, 224), # (0, 224, 224),
            "P": (255, 0, 255),
            #"p": (128, 0, 255),
            "p": (0, 192, 255),
            # "g": (0, 192, 0),
            # "N": (192, 192, 192),
            "N": (224, 224, 224), # (128, 128, 255),
            "I": (224, 224, 224), # (0, 255, 192),
            "{": (0, 0, 0),
            "<": (0, 0, 0),
            "W": (255, 255, 255)
        }.get(col[0], (128, 128, 128))
        if col[:4] in ["sj-f", "pl-t", "TTTT"]:
            rgb = (0, 192, 255) # (192, 64, 255)
        #border = (255, 255, 0) if col[0] == "{" else (255, 0, 255) if col[0] == "<" else (0, 0, 0)
        border = (0, 0, 0)
        draw.rectangle(((c + 1) * 256 + 32, (r + 1) * 256 + 32, (c + 1) * 256 + 223, (r + 1) * 256 + 223), fill=rgb, outline=border, width=4)
        if col in ["MJllf", "MJnfa", "MJ-t1", "MJ-t2", "MJ-t3", "MJ-t4", "SJcmf", "MIIII", "SIIII"]:
            rgb_ = (224, 224, 224) if col[0] == "M" else (192, 192, 192)
            for dy in range(4, 188):
                for dx in range(4, 188):
                    if (dx + dy) % 32 >= 16:
                        image.putpixel(((c + 1) * 256 + 32 + dx, (r + 1) * 256 + 32 + dy), rgb_)
        rgb_ = (255, 255, 0) if col[0] == "{" else (255, 0, 255) if col[0] == "<" else (0, 0, 0)
        key = col[:2].upper() + col[2:]
        text = names.get(key, col)
        draw_text((c, r), text, 160, rgb_)
        if key in locations:
            draw_text((c, r), locations[key], 160, rgb_, mode="location")
        if key in persons:
            draw_text((c, r), persons[key], 160, rgb_, mode="person")
        if key in notes:
            reqs = ", ".join([note for note in notes[key] if "require" in note or "available" in note or "optional" in note or "repeatable" in note or note == "Point of No Return"])
            notes_ = ", ".join([note for note in notes[key] if note not in reqs])
            if notes_:
                draw_text((c, r), notes_, 192, (128, 128, 128), mode="note")
            if reqs:
                draw_text((c, r), reqs, 192, (128, 128, 128), mode="req")
        arrow = lines[r][c * 7:c * 7 + 2]
        # HORIZONTAL BEFORE
        if arrow in ["->", "+>", "x>", "T>", "t>", "|>", "/>", "\>", ">>", " ~"]:
            a, b = (-4, 24) if arrow in ["|>", "/>", "\>", ">>", " ~"] or col in ["MJ-10", "WWlll", "WWrrr"] else (-32, 24)
            rgb = (128, 128, 128) if arrow in ["+>", "T>"] or col in ["MJ-01", "MJ-02", "MJ-03", "MJ-24", "{plr}", "MJ-26", "MJ-27", "MJ-31", "MJ-29", "SJ-35", "MJdfr", "MJaaw", "MJndf", "MJpog", "PLbsc", "PLtkm", "<cup>", "<pen>", "<swo>", "<wan>", "WWnnn", "WWooo", "WWrrr"] else (192, 192, 192)
            for dy in range(124, 132):
                for dx in range(a, b):
                    xy = ((c + 1) * 256 + dx, (r + 1) * 256 + dy)
                    if image.getpixel(xy) in [(255, 255, 255), (224, 224, 224), (192, 192, 192)]:
                        image.putpixel(xy, rgb)
            for dx in range(12):
                for dy in range((dx + 1) * 2):
                    image.putpixel(((c + 1) * 256 + 31 - dx, (r + 1) * 256 + 127 - dx + dy), rgb)
        if arrow == "=>":
            for dy in range(116, 124):
                for dx in range(-32, 32):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (192, 192, 192))
            for dy in range(132, 140):
                for dx in range(-32, 32):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (192, 192, 192))
        # VERTICAL BEFORE
        if arrow in ["/>", ">>", "x>", "+>", "T>"]:
            a, b = (124, 384)
            if col == "GG-01": b += 8 * 256
            if col == "GG-32": b += 5 * 256
            #if col == "sj-13": b += 1 * 256
            if col == "MJ-21": b += 3 * 256
            if col == "SJ-35": b += 3 * 256
            if col == "MJ-26": b += 3 * 256
            if col == "gghoh": b += 1 * 256
            rgb = (128, 128, 128) if arrow == "+>" or col in ["MJ-01", "MJ-21", "MJ-27", "SJ-35", "MJaaw", "PLbsc", "<cup>", "WWnnn"] else (192, 192, 192)
            for dy in range(a, b):
                for dx in range(-4, 4):
                    xy = ((c + 1) * 256 + dx, (r + 1) * 256 + dy)
                    if image.getpixel(xy) in [(255, 255, 255), (224, 224, 224), (192, 192, 192)]:
                        image.putpixel(xy, rgb)
        if arrow in ["\>", ">>", "x>", "+>", "t>"] or col == "PLded":
            a, b = (-128, 132)
            if col == "sjsow": a += 4
            if col == "NC-10": a += 4
            if col == "NC-01": a -= 14 * 256
            if col == "sj-1b": a -= 10 * 256
            #if col == "sj-13": a -= 1 * 256
            if col == "SJ-27": a -= 2 * 256
            if col == "MJ-24": a -= 3 * 256
            if col == "MJ-26": a -= 1 * 256
            if col == "GGsij": a -= 1 * 256
            if col == "PLded": a -= 7 * 256
            rgb = (128, 128, 128) if arrow in ["+>", "T>"] or col in ["MJ-03", "MJ-24", "MJ-29", "MJdfr", "MJpog", "{plr}", "PLded", "PLtkm", "<wan>", "WWooo"] else (192, 192, 192)
            for dy in range(a, b):
                for dx in range(-4, 4):
                    xy = ((c + 1) * 256 + dx, (r + 1) * 256 + dy)
                    if image.getpixel(xy) in [(255, 255, 255), (224, 224, 224), (192, 192, 192)]:
                        image.putpixel(xy, rgb)
        # MISSION WITHIN MISSION
        if arrow in ["vv", "^^"]:
            a, b = (224, 288) if arrow == "vv" else (-32, 32)
            for dy in range(a, b):
                for dx in range(124, 132):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (192, 192, 192))
        # HORIZONTAL AFTER
        try:
            cont = lines[r][c * 7 + 7]
        except IndexError:
            cont = ""
        if cont == "~" or col in ["MJ-08", "MJ-23", "MJ-t2", "PLfst"]:
            a, b = (224, 256 + 4)
            if col == "MJ-23": b += 2 * 256
            if col == "MJ-30": b += 1 * 256
            if col == "MJ-t3": b += 5 * 256
            if col == "MJ-t4": b += 4 * 256
            if col == "MJdfr": b += 2 * 256
            if col == "PLtph": b += 2 * 256
            rgb = (128, 128, 128) if col in ["MJ-08", "MJ-09", "MJ-23", "MJ-t2", "MJ-t3", "MJ-t4", "PLfst", "WWmmm", "WWppp", "WWqqq"] else (192, 192, 192)
            for dy in range(124, 132):
                for dx in range(a, b):
                    try:
                        image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), rgb)
                    except IndexError:
                        continue
        # PLUS BEFORE
        if (arrow in ["+>", "T>", "t>"] and col != "PLfss") or col == "MJ-26":
            ddx = -256 if col == "MJ-26" else 0
            for dy in range(112, 144):
                for dx in range(-16 + ddx, 16 + ddx):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (0, 0, 0))
            for dy in range(125, 131):
                for dx in range(-12 + ddx, 12 + ddx):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (254, 254, 254))
            for dy in range(116, 140):
                for dx in range(-3 + ddx, 3 + ddx):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (254, 254, 254))
        # EXCLAMATION MARK AFTER
        if col in ["MJ-02", "MJ-26", "MJcha", "PLlis", "PLfst", "PLtkm", "WWmmm"]:
            ddx = -256 if col == "MJ-02" else 0
            for dy in range(112, 144):
                for dx in range(240 + ddx, 272 + ddx):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (0, 0, 0))
            for dy in range(116, 130):
                for dx in range(253 + ddx, 259 + ddx):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (254, 254, 254))
            for dy in range(134, 140):
                for dx in range(253 + ddx, 259 + ddx):
                    image.putpixel(((c + 1) * 256 + dx, (r + 1) * 256 + dy), (254, 254, 254))


text = "Cyberpunk 2077 Mission Tree with Phantom Liberty".upper()
l, t, r, b = draw.textbbox((0, 0), text, font=font_36)
w, h = r - l, b - t
draw.multiline_text((36 * 256 + 146 - w / 2, 1 * 256 + 112 - h / 2), text, font=font_36, fill=(0, 0, 0), align="center")
text = "This diagram shows the unlock order of all jobs in the game."
text = "Public Domain. If you find an error, please report it at https://reddit.com/r/cyberpunkgame/comments/177pi79".upper()
l, t, r, b = draw.textbbox((0, 0), text, font=font_24)
w, h = r - l, b - t
draw.multiline_text((36 * 256 + 146 - w / 2, 10 * 256 + 120 - h / 2), text, font=font_24, fill=(0, 0, 0), align="center")
image.save("cyberpunk2077flowchart2.png")